package jp.gr.java_conf.ykhr.eclipse.plugin.junitreporter.main;

import java.io.File;
import java.util.Map;

import jp.gr.java_conf.ykhr.eclipse.plugin.junitreporter.JUnitReporterActivator;
import jp.gr.java_conf.ykhr.eclipse.plugin.junitreporter.ReportProperty;
import jp.gr.java_conf.ykhr.eclipse.plugin.junitreporter.ex.WriteException;

import org.eclipse.core.runtime.IPath;
import org.eclipse.core.runtime.IProgressMonitor;


public abstract class ReporterCommand implements IReporterCommand {
    
    protected Map testResults;
    protected ReportProperty prop;
    
    public ReporterCommand(Map testResults, ReportProperty prop) {
        this.testResults = testResults;
        this.prop = prop;
    }
    
    public static IReporterCommand getCommand(Map testResult, ReportProperty prop) {
        if (prop.isFileType()) {
            return new FileCommand(testResult, prop);
        }
        return new DirCommand(testResult, prop);
    }
    
    public String getName() {
        return "JUnit Reporter";
    }
    
    public void execute(IProgressMonitor monitor) throws WriteException {
        monitor.beginTask(getName(), 3);
        
        String output = getOutputPath(monitor);
        if (output == null) {
            return;
        }
        
        monitor.worked(1);
        if (monitor.isCanceled()) {
            return;
        }
        
        String xmlPath = createXml(monitor);
        monitor.worked(1);
        if (monitor.isCanceled()) {
            return;
        }
        
        createReport(output, xmlPath, monitor);
        monitor.worked(1);
        monitor.done();
    }
    
    protected void createReport(String output, String xmlPath, IProgressMonitor monitor) throws WriteException {
        monitor.setTaskName("create html");
        HtmlConverter converter = getHtmlConverter(xmlPath, output);
        converter.convert();
    }
    
    private String createXml(IProgressMonitor monitor) throws WriteException {
        monitor.setTaskName("create xml");
        String xmlPath = getXmlOutputPath();
        JUnitXmlWriter writer = new JUnitXmlWriter(xmlPath, testResults.values());
        writer.write();
        return xmlPath;
    }
    
    protected String getXmlOutputPath() {
        IPath stateLocation = JUnitReporterActivator.getDefault().getStateLocation();
        IPath xmlDestPath = stateLocation.append("junit").append("junit.xml");
        
        File xmlDest = xmlDestPath.toFile();
        File parent = xmlDest.getParentFile();
        if (!parent.exists()) {
            parent.mkdirs();
        }
        return xmlDest.getAbsolutePath();
    }
    
    protected abstract String getOutputPath(IProgressMonitor monitor);
    protected abstract HtmlConverter getHtmlConverter(String xmlPath, String output);
    
}
